//! Query utilities
//!
//! This module provides query-related utilities.
//! The main Query type is in the types module and has builder methods built-in.

// This module is kept for potential future query utilities
// Query itself has all the builder methods needed
